<?php

namespace Drupal\excel2webpage\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\excel2webpage\Entity\CustomExcelTable;
use Drupal\file\Entity\File;

/**
 * Form controller for the manage_inventory entity edit forms.
 *
 * @ingroup commerce_multivendor
 */
class CustomExcelTableForm extends CustomExcelTableMultiStepForm {

  /**
   * {@inheritdoc}.
   */
  public function getFormId() {
    return 'custom_excel_table_basic_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $current_path = \Drupal::service('path.current')->getPath();
    $entity_id = 0;
    $default_title = '';
    $default_excel = '';
    $default_description = '';
    $default_path = '';

    if (preg_match('/excel-table\/(\d+)/', $current_path, $matches)) {
      $entity_id = $matches[1];
      $entity = CustomExcelTable::load($entity_id);

      $default_title = $entity->getTitle();
      $default_excel = [$entity->get('excel_file')->getValue()[0]['target_id']];
      $default_description = $entity->get('description')->getValue()[0]['value'];
      $alias = \Drupal::entityTypeManager()->getStorage('path_alias')->loadByProperties([
        'path' => '/custom_excel_table/' . $entity->id(),
      ]);
      $default_path = $alias[array_key_first($alias)]->alias->value;

    }

    if (\Drupal::request()->query->get('entity_id')) {

      $entity_id = \Drupal::request()->query->get('entity_id');
      $entity = CustomExcelTable::load($entity_id);

      $default_title = $entity->getTitle();
      $default_excel = [$entity->get('excel_file')->getValue()[0]['target_id']];
      $default_description = $entity->get('description')->getValue()[0]['value'];
      $alias = \Drupal::entityTypeManager()->getStorage('path_alias')->loadByProperties([
        'path' => '/custom_excel_table/' . $entity->id(),
      ]);
      $default_path = $alias[array_key_first($alias)]->alias->value;
    }

    // Getting entity id.
    $form['title'] = [
      '#type' => 'textfield',
      '#title' => 'Title',
      '#required' => TRUE,
      '#default_value' => $default_title,
    ];

    $form['excel_file'] = [
      '#type' => 'managed_file',
      '#title' => t('File'),
      '#required' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['xls xlsx xlst'],
      ],
      '#default_value' => $default_excel,
    ];

    // If there is excel file uploaded.
    $form['description'] = [
      '#type' => 'textarea',
      '#title' => 'Description',
      '#default_value' => $default_description,
    ];

    $form['path'] = [
      '#type' => 'textfield',
      '#title' => 'Path',
      '#required' => TRUE,
      '#attributes' => ['placeholder' => '/excel-table'],
      '#default_value' => $default_path,
      '#description' => '<p>Specify an alternative path by which this data can be accessed. For example, type "/excel-table" when uploading excel file </p>',
    ];

    $form['entity_id'] = [
      '#type' => 'hidden',
      '#value' => $entity_id,
    ];

    $form['actions']['next'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#value' => $this->t('Next'),
    ];

    return $form;
  }

  /**
   * Validate form.
   *
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // @todo Change the autogenerated stub
    parent::validateForm($form, $form_state);

    $excel_entity_id = $form_state->getValue('entity_id');
    $source = '/custom_excel_table/' . $excel_entity_id;

    $url_path = $form_state->getValue('path');
    $url_first = substr($url_path, 0, 1);

    if ($url_first != '/') {
      $form_state->setErrorByName('path', 'Please prefix url with "/"');
    }
    $alias = \Drupal::entityTypeManager()->getStorage('path_alias')->loadByProperties([
      'alias' => $url_path,
    ]);

    if (!empty($alias) && $excel_entity_id == 0) {
      $form_state->setErrorByName('path', 'URL is already exist.');
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    \Drupal::request()->query->remove('destination');

    $excel_entity_id = $form_state->getValue('entity_id');
    $excel_file_id = $form_state->getValue('excel_file')[0];
    $excel_file = File::load($excel_file_id);

    // If its existing entity.
    if ($excel_entity_id > 0) {
      $excel_entity = CustomExcelTable::load($excel_entity_id);
      $excel_entity->set('title', $form_state->getValue('title'));
      $excel_entity->set('description', $form_state->getValue('description'));
      $excel_entity->set('excel_file', $excel_file);
      $excel_entity->set('path', $form_state->getValue('path'));
    }
    else {
      // Creating entity.
      $excel_entity = CustomExcelTable::create([
        'title' => $form_state->getValue('title'),
        'description' => $form_state->getValue('description'),
        'uid' => \Drupal::currentUser()->id(),
        'excel_file' => $excel_file,
        'path' => $form_state->getValue('path'),
      ]
      );
    }

    // Saving entity.
    $excel_entity->save();

    $form_state->setRedirect('entity.custom_excel_table.add_sheets', [], ['query' => ['entity_id' => $excel_entity->id()]]);
  }

}
